import { bucketName, minioClient } from '../minio.ts';
import { IncomingMessage, ServerResponse } from 'http';
import { filterKeys } from './http-proxy.ts';
import { getUserFromRequest } from '@/utils/get-user.ts';
import { UserPermission, Permission } from '@kevisual/permission';
import { getLoginUser } from '@/middleware/auth.ts';
import busboy from 'busboy';
import { getContentType } from '../get-content-type.ts';

const getAiProxy = async (
  req: IncomingMessage,
  res: ServerResponse,
  opts: {
    createNotFoundPage: (msg?: string) => any;
  },
) => {
  const { createNotFoundPage } = opts;
  const _u = new URL(req.url, 'http://localhost');

  const pathname = _u.pathname;
  const params = _u.searchParams;
  const password = params.get('p');
  let objectName = '';
  let owner = '';
  const { user, app } = getUserFromRequest(req);
  if (app === 'ai') {
    const version = params.get('version') || '1.0.0'; // root/ai
    objectName = pathname.replace(`/${user}/${app}/`, `${user}/${app}/${version}/`);
    owner = user;
  } else {
    objectName = pathname.replace(`/${user}/${app}/`, `${user}/`); // resources/root/
    owner = user;
  }
  try {
    const stat = await minioClient.statObject(bucketName, objectName);
    if (stat.size === 0) {
      createNotFoundPage('Invalid proxy url');
      return true;
    }
    const permissionInstance = new UserPermission({ permission: stat.metaData as Permission, owner: owner });
    const loginUser = await getLoginUser(req);
    const checkPermission = permissionInstance.checkPermissionSuccess({
      username: loginUser?.tokenUser?.username || '',
      password: password,
    });
    if (!checkPermission.success) {
      return createNotFoundPage('no permission');
    }
    const filterMetaData = filterKeys(stat.metaData, ['size', 'etag', 'last-modified']);
    const contentLength = stat.size;
    const etag = stat.etag;
    const lastModified = stat.lastModified.toISOString();
    const fileName = objectName.split('/').pop();

    const objectStream = await minioClient.getObject(bucketName, objectName);
    const headers = {
      'Content-Length': contentLength,
      etag,
      'last-modified': lastModified,
      'file-name': fileName,
      ...filterMetaData,
    };

    res.writeHead(200, {
      ...headers,
    });
    objectStream.pipe(res, { end: true });
    return true;
  } catch (error) {
    console.error(`Proxy request error: ${error.message}`);
    createNotFoundPage('Invalid ai proxy url');
    return false;
  }
};
export const getMetadata = (pathname: string) => {
  let meta: any = { 'app-source': 'user-app' };
  const isHtml = pathname.endsWith('.html');
  if (isHtml) {
    meta = {
      ...meta,
      'Content-Type': 'text/html; charset=utf-8',
      'Cache-Control': 'no-cache',
    };
  } else {
    meta = {
      ...meta,
      'Content-Type': getContentType(pathname),
      'Cache-Control': 'max-age=31536000, immutable',
    };
  }
  return meta;
};
export const postProxy = async (req: IncomingMessage, res: ServerResponse, opts: { createNotFoundPage: (msg?: string) => any }) => {
  const _u = new URL(req.url, 'http://localhost');

  const pathname = _u.pathname;
  const params = _u.searchParams;
  let objectName = '';
  let owner = '';
  const { user, app } = getUserFromRequest(req);
  if (app === 'ai') {
    const version = params.get('version') || '1.0.0'; // root/ai
    objectName = pathname.replace(`/${user}/${app}/`, `${user}/${app}/${version}/`);
    owner = user;
  } else {
    objectName = pathname.replace(`/${user}/${app}/`, `${user}/`); // resources/root/
    owner = user;
  }
  const loginUser = await getLoginUser(req);
  if (loginUser?.tokenUser?.username !== owner) {
    return opts?.createNotFoundPage?.('no permission');
  }
  const bb = busboy({ headers: req.headers });
  bb.on('file', async (name, file, info) => {
    try {
      await minioClient.putObject(bucketName, objectName, file, undefined, {
        ...getMetadata(pathname),
      });
      end({ success: true, name, info }, '上传成功', 200);
    } catch (error) {
      end({ error: error }, '上传失败', 500);
    }
  });
  const end = (data: any, message?: string, code = 200) => {
    res.writeHead(code, { 'Content-Type': 'application/json' });
    res.end(JSON.stringify({ code: code, data: data, message: message || 'success' }));
  };
  bb.on('finish', end);
  bb.on('error', (err) => {
    console.error('Busboy 错误:', err);
    end({ error: err }, '文件解析失败', 500);
  });

  req.pipe(bb);
};
export const aiProxy = async (
  req: IncomingMessage,
  res: ServerResponse,
  opts: {
    createNotFoundPage: (msg?: string) => any;
  },
) => {
  if (req.method === 'POST') {
    return postProxy(req, res, opts);
  }

  return getAiProxy(req, res, opts);
};
